package org.bdware.bdledger.api.grpc;

import com.google.protobuf.ByteString;
import io.grpc.Status;
import org.bdware.bdledger.api.grpc.pb.CommonProto.Block;
import org.bdware.bdledger.api.grpc.pb.CommonProto.Transaction;
import org.bdware.bdledger.api.grpc.pb.CommonProto.TransactionType;
import org.bdware.bdledger.api.grpc.pb.LedgerProto.GetLedgersResponse;
import org.junit.jupiter.api.*;

import java.util.Arrays;
import java.util.List;
import java.util.concurrent.ExecutionException;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

import static org.junit.jupiter.api.Assertions.assertEquals;

@DisplayName("Transaction Ledger tests")
class ClientTests {

  private static final String ledger = "test";
  private static final int blockCount = 2018;
  private static final int transactionCount = 2020;

  private static final String blockHashStr = "deadc0dedeadc0dedeadc0dedeadc0dedeadc0de";
  private static final ByteString blockHash =
      ByteString.copyFrom(Utils.hexStringToByteArray(blockHashStr));
  private static final String[] hashStrs = {
      "0404040404040404040404040404040404040404",
      "1313131313131313131313131313131313131313",
      "5252525252525252525252525252525252525252"
  };
  private static final List<ByteString> hashes =
      Arrays.stream(hashStrs)
          .map(s -> ByteString.copyFrom(Utils.hexStringToByteArray(s)))
          .collect(Collectors.toList());
  private static final List<ByteString> parentHashes = hashes;
  private static final List<ByteString> witnesses = hashes;
  private static final long timestamp = 2018050400000L;
  private static final long size = 20180504L;
  private static final int blockTransactionCount = 1000;
  private static final ByteString transactionsRoot =
      ByteString.copyFrom(Utils.hexStringToByteArray("babefacebabefacebabefacebabefacebabeface"));

  private static final int index = 0;
  private static final ByteString from =
      ByteString.copyFrom(Utils.hexStringToByteArray("f00dcafef00dcafef00dcafef00dcafef00dcafe"));
  private static final long nonce = 2018L;
  private static final ByteString to =
      ByteString.copyFrom(Utils.hexStringToByteArray("feedbabefeedbabefeedbabefeedbabefeedbabe"));
  private static final ByteString data =
      ByteString.copyFrom(Utils.hexStringToByteArray("deadbeef"));
  private static final ByteString v = ByteString.copyFrom(Utils.hexStringToByteArray("25"));
  private static final ByteString r =
      ByteString.copyFrom(
          Utils.hexStringToByteArray(
              "1b5e176d927f8e9ab405058b2d2457392da3e20f328b16ddabcebc33eaac5fea"));
  private static final ByteString s =
      ByteString.copyFrom(
          Utils.hexStringToByteArray(
              "4ba69724e8f69de52f0125ad8b3c5c2cef33019bac3249e2c0a2192766d1721c"));
  private static final String txHashStr = "0404040404040404040404040404040404040404";
  private static final ByteString txHash1 =
      ByteString.copyFrom(Utils.hexStringToByteArray(txHashStr));
  private static final ByteString txHash2 =
      ByteString.copyFrom(Utils.hexStringToByteArray("1313131313131313131313131313131313131313"));

  private static final Block block =
      Block.newBuilder()
          .setHash(blockHash)
          .addAllParentHashes(parentHashes)
          .addAllWitnesses(witnesses)
          .setTimestamp(timestamp)
          .setSize(size)
          .setTransactionCount(blockTransactionCount)
          .setTransactionsRoot(transactionsRoot)
          .addAllTransactionHashes(Arrays.asList(txHash1, txHash2))
          .build();
  private static final Transaction tx =
      Transaction.newBuilder()
          .setBlockHash(blockHash)
          .setIndex(index)
          .setHash(txHash1)
          .setType(TransactionType.RECORD)
          .setFrom(from)
          .setNonce(nonce)
          .setTo(from)
          .setData(data)
          .setV(v)
          .setR(r)
          .setS(s)
          .build();

  private static Client client;

//  @BeforeAll
//  static void initAll() {
//    client = new Client("localhost", 10000);
//  }
//
//  @Test
//  @DisplayName("ClientVersion#1")
//  void clientVersion1() throws ExecutionException, InterruptedException {
//    Assertions.assertEquals(
//        "TxLedgerGo/v0.0.1alpha/darwin/go1.11", client.clientVersion().get().getVersion());
//  }
//
//  @Test
//  @DisplayName("CreateLedger#1")
//  void createLedger1() throws ExecutionException, InterruptedException {
//    Assertions.assertTrue(client.createLedger(ledger).get().getOk());
//  }
//
//  @Test
//  @DisplayName("CreateLedger#2")
//  void createLedger2() {
//    Throwable e = Assertions.assertThrows(Exception.class, () -> client.createLedger("").get());
//    Status s = Status.fromThrowable(e);
//    Assertions.assertEquals(Status.Code.INVALID_ARGUMENT, s.getCode());
//    Assertions.assertEquals("name must not be empty", s.getDescription());
//  }
//
//  @Test
//  @DisplayName("GetLedgers#1")
//  void getLedgers1() throws ExecutionException, InterruptedException {
//    GetLedgersResponse r = client.getLedgers().get();
//    assertEquals(3, r.getLedgersCount());
//    String[] expected = new String[]{"first", "second", "third"};
//    Assertions.assertAll(
//        IntStream.range(0, 3).boxed().map(i -> () -> assertEquals(expected[i], r.getLedgers(i))));
//  }
//
//  @Test
//  @DisplayName("SendTransaction#1")
//  void sendTransaction1() throws ExecutionException, InterruptedException {
//    String hash =
//        Utils.byteArrayToHexString(
//            client
//                .sendTransaction(
//                    ledger,
//                    TransactionType.MESSAGE,
//                    "f00dcafef00dcafef00dcafef00dcafef00dcafe",
//                    "feedbabefeedbabefeedbabefeedbabefeedbabe",
//                    Utils.hexStringToByteArray("deadbeef"))
//                .get()
//                .getHash()
//                .toByteArray());
//    Assertions.assertEquals("d15ea5edd15ea5edd15ea5edd15ea5edd15ea5ed", hash);
//  }
//
//  @Test
//  @DisplayName("SendTransaction#2")
//  void sendTransaction2() {
//    Throwable e =
//        Assertions.assertThrows(
//            Exception.class,
//            () ->
//                client
//                    .sendTransaction(ledger, TransactionType.MESSAGE, null, "50bada55", null)
//                    .get());
//    Status s = Status.fromThrowable(e);
//    Assertions.assertEquals(Status.Code.INVALID_ARGUMENT, s.getCode());
//    Assertions.assertEquals("Multiple invalid arguments", s.getDescription());
//  }
//
//  @Test
//  @DisplayName("GetBlockByHash#1")
//  void getBlockByHash1() throws ExecutionException, InterruptedException {
//    Block b = client.getBlockByHash(ledger, blockHashStr, false).get().getBlock();
//    assertEquals(block, b);
//  }
//
//  // TODO
//  @Test
//  @DisplayName("getBlocks#1")
//  void getBlocks1() throws ExecutionException, InterruptedException {
//    client.getBlocks(ledger, 0, null).get();
//  }
//
//  // TODO
//  @Test
//  @DisplayName("getBlocks#2")
//  void getBlocks2() throws ExecutionException, InterruptedException {
//    client.getBlocks(ledger, 0, 0, null).get();
//  }
//
//  @Test
//  @DisplayName("getBlocks#3")
//  void getBlocks3() {
//    Throwable e = Assertions.assertThrows(Exception.class, () -> client.getBlocks("", 0, 0, null).get());
//    Status s = Status.fromThrowable(e);
//    Assertions.assertEquals(Status.Code.INVALID_ARGUMENT, s.getCode());
//    Assertions.assertEquals("ledger must not be empty", s.getDescription());
//  }
//
//  @Test
//  @DisplayName("CountBlocks#1")
//  void blockNumber1() throws ExecutionException, InterruptedException {
//    Assertions.assertEquals(blockCount, client.countBlocks(ledger).get().getCount());
//  }
//
//  @Test
//  @DisplayName("CountBlocks#2")
//  void blockNumber2() {
//    Throwable e = Assertions.assertThrows(Exception.class, () -> client.countBlocks("").get());
//    Status s = Status.fromThrowable(e);
//    Assertions.assertEquals(Status.Code.INVALID_ARGUMENT, s.getCode());
//    Assertions.assertEquals("ledger must not be empty", s.getDescription());
//  }
//
//  @Test
//  @DisplayName("GetTransactionByHash#1")
//  void getTransactionByHash1() throws ExecutionException, InterruptedException {
//    Transaction t = client.getTransactionByHash(ledger, txHashStr).get().getTransaction();
//    assertEquals(tx, t);
//  }
//
//  @Test
//  @DisplayName("GetTransactionByBlockHashAndIndex#1")
//  void GetTransactionByBlockHashAndIndex1() throws ExecutionException, InterruptedException {
//    Transaction t = client.getTransactionByBlockHashAndIndex(ledger, blockHashStr, index).get().getTransaction();
//    assertEquals(tx, t);
//  }
//
//  // TODO
//  @Test
//  @DisplayName("getTransactions#1")
//  void getTransactions1() throws ExecutionException, InterruptedException {
//    client.getTransactions(ledger, 0).get();
//  }
//
//  // TODO
//  @Test
//  @DisplayName("getTransactions#2")
//  void getTransactions2() throws ExecutionException, InterruptedException {
//    client.getTransactions(ledger, 0, 0).get();
//  }
//
//  @Test
//  @DisplayName("getTransactions#3")
//  void getTransactions3() {
//    Throwable e = Assertions.assertThrows(Exception.class, () -> client.getTransactions("", 0, 0).get());
//    Status s = Status.fromThrowable(e);
//    Assertions.assertEquals(Status.Code.INVALID_ARGUMENT, s.getCode());
//    Assertions.assertEquals("ledger must not be empty", s.getDescription());
//  }
//
//  @Test
//  @DisplayName("CountTransactions#1")
//  void CountTransactions1() throws ExecutionException, InterruptedException {
//    Assertions.assertEquals(transactionCount, client.countTransactions(ledger).get().getCount());
//  }
//
//  @Test
//  @DisplayName("CountTransactions#2")
//  void CountTransactions2() {
//    Throwable e = Assertions.assertThrows(Exception.class, () -> client.countTransactions("").get());
//    Status s = Status.fromThrowable(e);
//    Assertions.assertEquals(Status.Code.INVALID_ARGUMENT, s.getCode());
//    Assertions.assertEquals("ledger must not be empty", s.getDescription());
//  }
//
//  @AfterAll
//  static void teadDwonAll() throws InterruptedException {
//    client.shutdown();
//  }
}
